@@ -18,12 +18,11 @@ class InvalidAPIKey(BaseFredAPIError):
         )
 
 
-class APIKeyNotFoundError(BaseFredAPIError):
-    """Error raised when FRED_API_KEY not found in the environment."""
-
+class APIKeyNotFound(BaseFredAPIError):
     def __init__(self):
+        """Error raised when FRED_API_KEY not found in the environment."""
         super().__init__(
-            """API key not found. Either set a FRED_API_KEY environment variable or pass your API key to the api_key parameter"""
+            """API key not found. Either set a FRED_API_KEY environment variable or pass your API key to the `api_key` parameter."""
         )
 
 
