@@ -1,13 +1,14 @@
-"""The series module provides functions to request data from the `FRED API Sources endpoints <https://fred.stlouisfed.org/docs/api/fred/#Series>'_."""
+"""The `series` module provides functions to request data from the [FRED API Sources endpoints](https://fred.stlouisfed.org/docs/api/fred/#Series)."""
 
 import webbrowser
 from typing import List, Literal, Optional
 
 import pandas as pd
 from frozendict import frozendict
-from pydantic import BaseModel, Extra, PositiveInt
+from pydantic import BaseModel, ConfigDict, PositiveInt
 
 from ._base import _get_request
+from .utils import _convert_pydantic_model_to_frozen_dict
 from .utils._common_type_hints import (
     ApiKeyType,
     JsonOrPdType,
@@ -25,6 +26,8 @@ _latest_realtime_end: str = "9999-12-31"
 class SeriesApiParameters(BaseModel):
     """Represents the parameters accepted by the FRED Series endpoints."""
 
+    model_config = ConfigDict(extra="allow")
+
     series_id: Optional[str] = None
     realtime_start: Optional[str] = None
     realtime_end: Optional[str] = None
@@ -70,13 +73,12 @@ class SeriesApiParameters(BaseModel):
     output_type: Optional[Literal["1", "2", "3", "4", 1, 2, 3, 4]] = None
     vintage_dates: Optional[str] = None
 
-    class Config:
-        extra = Extra.allow
-
 
 class SeriesSearchParameters(BaseModel):
     """Represents the parameters accepted by the FRED Series Search endpoints."""
 
+    model_config = ConfigDict(extra="allow")
+
     search_text: Optional[str] = None
     search_type: Optional[Literal["full_text", "series_id"]] = None
     realtime_start: Optional[str] = None
@@ -101,19 +103,18 @@ class SeriesSearchParameters(BaseModel):
         ]
     ] = None
     sort_order: Optional[Literal["asc", "desc"]] = None
-    filter_variable: Optional[
-        Literal["frequency", "units", "seasonal_adjustment"]
-    ] = None
+    filter_variable: Optional[Literal["frequency", "units", "seasonal_adjustment"]] = (
+        None
+    )
     filter_value: Optional[str] = None
     tag_names: Optional[str] = None
     exclude_tag_names: Optional[str] = None
 
-    class Config:
-        extra = Extra.allow
-
 
 class SeriesInfo(BaseModel):
-    """Represents metadata about an economics data series. https://fred.stlouisfed.org/docs/api/fred/series.html."""
+    """Represents metadata about an economics data series. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series.html)."""
+
+    model_config = ConfigDict(extra="allow")
 
     id: str
     realtime_start: str
@@ -136,20 +137,17 @@ class SeriesInfo(BaseModel):
         """Open the FRED webpage for the given series."""
         webbrowser.open(f"{self._base_url}/{self.id}", new=2)
 
-    class Config:
-        extra = Extra.allow
-
 
 def get_series_info(
     series_id: str, api_key: ApiKeyType = None, **kwargs: KwargsType
 ) -> SeriesInfo:
-    """Get an economic data series information by ID. https://fred.stlouisfed.org/docs/api/fred/series.html.
+    """Get an economic data series information by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -158,12 +156,15 @@ def get_series_info(
     -------
     SeriesInfo
         An instance of SeriesInfo.
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     response = _get_request(
         api_key=api_key,
         endpoint="series",
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
     return SeriesInfo(**response["seriess"][0])
 
@@ -171,13 +172,13 @@ def get_series_info(
 def get_series_categories(
     series_id: str, api_key: ApiKeyType = None, **kwargs: KwargsType
 ) -> JsonType:
-    """Get the categories for an economic data series by ID. https://fred.stlouisfed.org/docs/api/fred/series_categories.html.
+    """Get the categories for an economic data series by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_categories.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/categories`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -186,12 +187,15 @@ def get_series_categories(
     -------
     dict
         Dictionary representing the json response.
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     return _get_request(
         api_key=api_key,
         endpoint="series/categories",
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
 
@@ -201,13 +205,13 @@ def get_series(
     return_format: ReturnFmtType = "pandas",
     **kwargs: KwargsType,
 ) -> JsonOrPdType:
-    """Get the observations or data values for an economic data series by ID. https://fred.stlouisfed.org/docs/api/fred/series_observations.html.
+    """Get the observations or data values for an economic data series by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_observations.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : Literal[json, pandas] | ReturnFormat, optional
         Define how to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -218,14 +222,17 @@ def get_series(
     -------
     dict | pd.DataFrame
         Either a dictionary representing the json response or a pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     response = _get_request(
         api_key=api_key,
         endpoint="series/observations",
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
     if return_format == ReturnFormat.pandas:
@@ -237,13 +244,13 @@ def get_series(
 def get_series_releases(
     series_id: str, api_key: ApiKeyType = None, **kwargs: KwargsType
 ) -> JsonType:
-    """Get the FRED release for an economic data series by ID. https://fred.stlouisfed.org/docs/api/fred/series_release.html.
+    """Get the FRED release for an economic data series by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_release.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/releases`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -252,25 +259,28 @@ def get_series_releases(
     -------
     dict
         Dictionary representing the json response.
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     return _get_request(
         endpoint="series/release",
         api_key=api_key,
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
 
 def get_series_tags(
     series_id: str, api_key: ApiKeyType = None, **kwargs: KwargsType
 ) -> JsonType:
-    """Get the FRED tags for an economic data series by ID. https://fred.stlouisfed.org/docs/api/fred/series_tags.html.
+    """Get the FRED tags for an economic data series by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_tags.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/tags`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -279,25 +289,28 @@ def get_series_tags(
     -------
     dict
         Dictionary representing the json response.
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     return _get_request(
         endpoint="series/tags",
         api_key=api_key,
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
 
 def get_series_updates(
     series_id: str, api_key: ApiKeyType = None, **kwargs: KwargsType
 ) -> JsonType:
-    """Get the FRED updates for an economic data series by ID. https://fred.stlouisfed.org/docs/api/fred/series_updates.html.
+    """Get the FRED updates for an economic data series by ID. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_updates.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/updates`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -306,12 +319,15 @@ def get_series_updates(
     -------
     dict
         Dictionary representing the json response.
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     return _get_request(
         endpoint="series/updates",
         api_key=api_key,
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
 
@@ -320,13 +336,13 @@ def get_series_vintagedates(
 ) -> List[str]:
     """Get the dates in history when a series' data values were revised or new data values were released.
 
-    Vintage dates are the release dates for a series excluding release dates when the data did not change. https://fred.stlouisfed.org/docs/api/fred/series_vintagesdates.html.
+    Vintage dates are the release dates for a series excluding release dates when the data did not change. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_vintagesdates.html).
 
     Parameters
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     **kwargs : dict, optional
         Additional parameters to FRED API ``series/vintagedates`` endpoint. Refer to the FRED documentation for a list of all possible parameters.
@@ -335,12 +351,15 @@ def get_series_vintagedates(
     -------
     List[str]
         List of strings representing the the available vintage dates
+
     """
-    params = SeriesApiParameters(series_id=series_id, **kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(series_id=series_id, **kwargs)
+    )
     response = _get_request(
         endpoint="series/vintagedates",
         api_key=api_key,
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
     return response["vintage_dates"]
 
@@ -357,7 +376,7 @@ def get_series_all_releases(
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -368,6 +387,7 @@ def get_series_all_releases(
     -------
     dict | pd.DataFrame
         Either a dictionary representing the json response or a pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
@@ -376,16 +396,18 @@ def get_series_all_releases(
     _ = kwargs.pop("realtime_start", None)
     _ = kwargs.pop("realtime_end", None)
 
-    params = SeriesApiParameters(
-        realtime_start=_earliest_realtime_start,
-        realtime_end=_latest_realtime_end,
-        **kwargs,
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(
+            realtime_start=_earliest_realtime_start,
+            realtime_end=_latest_realtime_end,
+            **kwargs,
+        )
     )
     return get_series(
         series_id=series_id,
         api_key=api_key,
         return_format=return_format,
-        **params.dict(exclude_none=True),
+        **params,
     )
 
 
@@ -403,7 +425,7 @@ def get_series_initial_release(
     ----------
     series_id : str
         Series id of interest.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -414,6 +436,7 @@ def get_series_initial_release(
     -------
     dict | pd.DataFrame
         Either a dictionary representing the json response or a pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
@@ -422,14 +445,16 @@ def get_series_initial_release(
     _ = kwargs.pop("realtime_start", None)
     _ = kwargs.pop("output_type", None)
 
-    params = SeriesApiParameters(
-        realtime_start=_earliest_realtime_start, output_type=4, **kwargs
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(
+            realtime_start=_earliest_realtime_start, output_type=4, **kwargs
+        )
     )
     return get_series(
         series_id=series_id,
         api_key=api_key,
         return_format=return_format,
-        **params.dict(exclude_none=True),
+        **params,
     )
 
 
@@ -451,7 +476,7 @@ def get_series_asof_date(
         Series id of interest.
     date : str
         Include only data revisions made on or before this date.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -462,6 +487,7 @@ def get_series_asof_date(
     -------
     dict | pd.DataFrame
         Either a dictionary representing the json response or a pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
@@ -470,14 +496,16 @@ def get_series_asof_date(
     _ = kwargs.pop("realtime_start", None)
     _ = kwargs.pop("realtime_end", None)
 
-    params = SeriesApiParameters(
-        realtime_start=_earliest_realtime_start, realtime_end=date, **kwargs
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesApiParameters(
+            realtime_start=_earliest_realtime_start, realtime_end=date, **kwargs
+        )
     )
     return get_series(
         series_id=series_id,
         api_key=api_key,
         return_format=return_format,
-        **params.dict(exclude_none=True),
+        **params,
     )
 
 
@@ -488,17 +516,17 @@ def search_series(
     return_format: ReturnFmtType = "pandas",
     **kwargs: KwargsType,
 ) -> JsonOrPdType:
-    """Get economic data series that match search text. `FRED docs <https://fred.stlouisfed.org/docs/api/fred/series_search.html>`_.
+    """Get economic data series that match search text. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_search.html).
 
     Parameters
     ----------
     search_text : str
         The text to match against.
-    api_key : str | None
+    api_key : str | None, optional, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     search_type : Literal["full_text", "series_id"]
         Defines which type of search to preform. One of the following strings: 'full_text', 'series_id'.
-        Parameter docs: https://fred.stlouisfed.org/docs/api/fred/series_search.html#search_type.
+        [Parameter docs](https://fred.stlouisfed.org/docs/api/fred/series_search.html#search_type).
     return_format : : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
     **kwargs : dict, optional
@@ -508,18 +536,21 @@ def search_series(
     -------
     dict | pd.DataFrame
         Either a pandas dataframe or json. Defaults to pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
-    params = SeriesSearchParameters(
-        search_text=search_text,
-        search_type=search_type,
-        **kwargs,
+    params = _convert_pydantic_model_to_frozen_dict(
+        SeriesSearchParameters(
+            search_text=search_text,
+            search_type=search_type,
+            **kwargs,
+        )
     )
     response = _get_request(
         endpoint="series/search",
         api_key=api_key,
-        params=frozendict(params.dict(exclude_none=True)),
+        params=params,
     )
 
     if return_format == ReturnFormat.pandas:
@@ -533,13 +564,13 @@ def search_series_tags(
     return_format: ReturnFmtType = "pandas",
     **kwargs: KwargsType,
 ) -> JsonOrPdType:
-    """Get the FRED tags for a series search. https://fred.stlouisfed.org/docs/api/fred/release_related_tags.html.
+    """Get the FRED tags for a series search. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/release_related_tags.html).
 
     Parameters
     ----------
     search_text : str
         The text to match against.
-    api_key : str | None
+    api_key : str | None, optional, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -550,23 +581,24 @@ def search_series_tags(
     -------
     dict | pd.DataFrame
         Either a pandas dataframe or json. Defaults to pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
-    params = SeriesSearchParameters(**kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(SeriesSearchParameters(**kwargs))
     response = _get_request(
         endpoint="series/search/tags",
         api_key=api_key,
         params=frozendict(
             {
                 "series_search_text": search_text,
-                **params.dict(exclude_none=True),
+                **params,
             }
         ),
     )
 
     if return_format == ReturnFormat.pandas:
-        return pd.DataFrame.from_dict(response["tags"])
+        return _convert_to_pandas(response["tags"])
     return response
 
 
@@ -577,7 +609,7 @@ def search_series_related_tags(
     return_format: ReturnFmtType = "pandas",
     **kwargs,
 ) -> JsonOrPdType:
-    """Get the related FRED tags for one or more FRED tags matching a series search. https://fred.stlouisfed.org/docs/api/fred/series_search_related_tags.html.
+    """Get the related FRED tags for one or more FRED tags matching a series search. [Endpoint documentation](https://fred.stlouisfed.org/docs/api/fred/series_search_related_tags.html).
 
     Parameters
     ----------
@@ -585,7 +617,7 @@ def search_series_related_tags(
         The text to match against.
     tag_names : str
         A semicolon delimited list of tag names that series match all of.
-    api_key : str | None
+    api_key : str | None, optional
         FRED API key. Defaults to None. If None, will search for FRED_API_KEY in environment variables.
     return_format : : Literal["json", "pandas"] | ReturnFormat, optional
         In what format to return the response. Must be either 'json' or 'pandas'. Defaults to 'pandas'.
@@ -596,10 +628,11 @@ def search_series_related_tags(
     -------
     dict | pd.DataFrame
         Either a pandas dataframe or json. Defaults to pandas dataframe.
+
     """
     return_format = ReturnFormat(return_format)
 
-    params = SeriesSearchParameters(**kwargs)
+    params = _convert_pydantic_model_to_frozen_dict(SeriesSearchParameters(**kwargs))
     response = _get_request(
         endpoint="series/search/related_tags",
         api_key=api_key,
@@ -607,7 +640,7 @@ def search_series_related_tags(
             {
                 "series_search_text": search_text,
                 "tag_names": tag_names,
-                **params.dict(exclude_none=True),
+                **params,
             }
         ),
     )
